# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/auth_accesstoken.ipynb.

# %% auto 0
__all__ = [
    "get_access_tokens",
    "AccessToken_GenerationError",
    "generate_access_token",
    "AccessToken_RevokeError",
    "revoke_access_token",
]

# %% ../../nbs/routes/auth_accesstoken.ipynb 2
import datetime as dt
import time

from dataclasses import dataclass, field
import httpx

import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.get_data as gd
import domolibrary.client.DomoAuth as dmda
import domolibrary.client.DomoError as de

# import domolibrary.client.Logger as lg


# %% ../../nbs/routes/auth_accesstoken.ipynb 5
@gd.route_function
async def get_access_tokens(
    auth: dmda.DomoAuth,
    debug_api: bool = False,
    debug_num_stacks_to_drop=1,
    parent_class=None,
    session: httpx.AsyncClient = None,
    return_raw: bool = False,
) -> rgd.ResponseGetData:

    url = f"https://{auth.domo_instance}.domo.com/api/data/v1/accesstokens"

    res = await gd.get_data(
        url=url,
        method="GET",
        auth=auth,
        debug_api=debug_api,
        parent_class=parent_class,
        num_stacks_to_drop=debug_num_stacks_to_drop,
    )

    if return_raw:
        return res

    [
        token.update({"expires": dt.datetime.utcfromtimestamp(token["expires"] / 1000)})
        for token in res.response
    ]

    return res


# %% ../../nbs/routes/auth_accesstoken.ipynb 7
def generate_expiration_unixtimestamp(
    duration_in_days: int = 90, debug_prn: bool = False
):

    today = dt.datetime.today()
    expiration_date = today + dt.timedelta(days=duration_in_days)

    if debug_prn:
        print(f"expiration_date is {duration_in_days} from today {expiration_date}")

    return int(time.mktime(expiration_date.timetuple()) * 1000)


# %% ../../nbs/routes/auth_accesstoken.ipynb 9
class AccessToken_GenerationError(de.DomoError):
    def __init__(
        self,
        user_id,
        domo_instance,
        parent_class,
        function_name,
        message=None,
    ):
        super().__init__(
            domo_instance=domo_instance,
            message=message or f"failure to generate access_token for {user_id}",
            function_name=function_name,
            parent_class=parent_class,
        )


@gd.route_function
async def generate_access_token(
    auth: dmda.DomoAuth,
    token_name: str,
    user_id,
    duration_in_days: 90,
    debug_api: bool = False,
    debug_num_stacks_to_drop=1,
    parent_class=None,
    session: httpx.AsyncClient = None,
) -> rgd.ResponseGetData:

    url = f"https://{auth.domo_instance}.domo.com/api/data/v1/accesstokens"

    expiration_timestamp = generate_expiration_unixtimestamp(
        duration_in_days=duration_in_days
    )

    body = {"name": token_name, "ownerId": user_id, "expires": expiration_timestamp}

    res = await gd.get_data(
        url=url,
        method="POST",
        body=body,
        auth=auth,
        debug_api=debug_api,
        parent_class=parent_class,
        num_stacks_to_drop=debug_num_stacks_to_drop,
    )

    if res.status == 400:
        raise AccessToken_GenerationError(
            user_id=user_id,
            message=f"unable to generate access_token for {user_id} || did you pass a valid user_id",
            domo_instance=auth.domo_instance,
            parent_class=parent_class,
            function_name=res.traceback_details.function_name,
        )

    if not res.is_success or not res.response["token"]:
        raise AccessToken_GenerationError(
            user_id=user_id,
            domo_instance=auth.domo_instance,
            parent_class=parent_class,
            function_name=res.traceback_details.function_name,
        )

    return res


# %% ../../nbs/routes/auth_accesstoken.ipynb 11
class AccessToken_RevokeError(de.DomoError):
    def __init__(
        self,
        access_token_id,
        domo_instance,
        parent_class,
        function_name,
    ):
        super().__init__(
            domo_instance=domo_instance,
            message=f"failure to revoke token {access_token_id}",
            function_name=function_name,
            parent_class=parent_class,
        )


@gd.route_function
async def revoke_access_token(
    auth: dmda.DomoAuth,
    access_token_id: int,
    debug_api: bool = False,
    debug_num_stacks_to_drop=1,
    parent_class=None,
    session: httpx.AsyncClient = None,
):
    url = f"https://{auth.domo_instance}.domo.com/api/data/v1/accesstokens/{access_token_id}"

    res = await gd.get_data(
        url=url,
        method="DELETE",
        auth=auth,
        debug_api=debug_api,
        parent_class=parent_class,
        num_stacks_to_drop=debug_num_stacks_to_drop,
    )

    if not res.is_success:
        raise AccessToken_RevokeError(
            access_token_id=access_token_id,
            domo_instance=auth.domo_instance,
            parent_class=parent_class,
            function_name=res.traceback_details.function_Name,
        )

    return res



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/sandbox.ipynb.

# %% auto 0
__all__ = [
    "get_is_allow_same_instance_promotion_enabled",
    "Sandbox_ToggleSameInstancePromotion_Error",
    "toggle_allow_same_instance_promotion",
    "get_shared_repos",
    "get_repo_from_id",
]

# %% ../../nbs/routes/sandbox.ipynb 2
import httpx

import domolibrary.client.get_data as gd
import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.DomoAuth as dmda
import domolibrary.client.DomoError as de


# %% ../../nbs/routes/sandbox.ipynb 4
@gd.route_function
async def get_is_allow_same_instance_promotion_enabled(
    auth: dmda.DomoAuth,
    session: httpx.AsyncClient = None,
    return_raw: bool = False,
    debug_num_stacks_to_drop: int = 1,
    debug_api: bool = False,
    parent_class: str = None,
):
    url = f"https://{auth.domo_instance}.domo.com/api/version/v1/settings"

    res = await gd.get_data(
        auth=auth,
        method="GET",
        url=url,
        session=session,
        debug_api=debug_api,
        num_stacks_to_drop=debug_num_stacks_to_drop,
        parent_class=parent_class,
    )

    if return_raw:
        return res

    res.response = {
        "name": "allow_same_instance_promotion",
        "is_enabled": res.response["allowSelfPromotion"],
    }

    return res


# %% ../../nbs/routes/sandbox.ipynb 6
class Sandbox_ToggleSameInstancePromotion_Error(de.DomoError):
    def __init__(
        self, domo_instance, message, status, parent_class=None, function_name=None
    ):
        super().__init__(
            self,
            domo_instance=domo_instance,
            status=status,
            parent_class=parent_class,
            message=message,
            function_name=function_name,
        )


@gd.route_function
async def toggle_allow_same_instance_promotion(
    is_enabled: bool,
    auth: dmda.DomoAuth,
    session: httpx.AsyncClient = None,
    debug_num_stacks_to_drop: int = 1,
    debug_api: bool = False,
    parent_class: str = None,
):
    url = f"https://{auth.domo_instance}.domo.com/api/version/v1/settings"

    body = {"allowSelfPromotion": is_enabled}

    res = await gd.get_data(
        auth=auth,
        method="POST",
        url=url,
        body=body,
        session=session,
        debug_api=debug_api,
        num_stacks_to_drop=debug_num_stacks_to_drop,
    )

    if not res.is_success:
        raise Sandbox_ToggleSameInstancePromotion_Error(
            domo_instance=auth.domo_instance,
            message=res.response,
            status=res.status,
            parent_class=parent_class,
            function_name=res.traceback_details.function_name,
        )

    return res


# %% ../../nbs/routes/sandbox.ipynb 8
@gd.route_function
async def get_shared_repos(
    auth: dmda.DomoAuth,
    session: httpx.AsyncClient = None,
    parent_class: str = None,
    debug_api: bool = False,
    debug_num_stacks_to_drop: bool = False,
) -> rgd.ResponseGetData:

    url = f"https://{auth.domo_instance}.domo.com/api/version/v1/repositories/search"

    body = {
        "query": {
            "offset": 0,
            "limit": 50,
            "fieldSearchMap": {},
            "sort": "lastCommit",
            "order": "desc",
            "filters": {"userId": None},
            "dateFilters": {},
        },
        "shared": False,
    }

    res = await gd.get_data(
        auth=auth,
        method="POST",
        url=url,
        body=body,
        session=session,
        debug_api=debug_api,
        parent_class=parent_class,
        num_stacks_to_drop=debug_num_stacks_to_drop,
    )

    return res


# %% ../../nbs/routes/sandbox.ipynb 11
@gd.route_function
async def get_repo_from_id(
    auth: dmda.DomoFullAuth,
    repository_id: str,
    debug_api: bool = False,
    debug_num_stacks_to_drop: int = 1,
    parent_class: str = None,
    session: httpx.AsyncClient = None,
) -> rgd.ResponseGetData:

    url = f"https://{auth.domo_instance}.domo.com/api/version/v1/repositories/{repository_id}"

    return await gd.get_data(
        auth=auth,
        method="GET",
        url=url,
        parent_class=parent_class,
        debug_api=debug_api,
        num_stacks_to_drop=debug_num_stacks_to_drop,
        session=session,
    )



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/publish.ipynb.

# %% auto 0
__all__ = [
    "search_publications",
    "get_publication_by_id",
    "generate_publish_body",
    "create_publish_job",
    "udpate_publish_job",
    "get_subscription_summaries",
    "get_subscription_invititations",
    "accept_invite_by_id",
    "accept_invite_by_id_v2",
    "refresh_publish_jobs",
]

# %% ../../nbs/routes/publish.ipynb 2
import httpx

import domolibrary.client.get_data as gd
import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.DomoAuth as dmda


# %% ../../nbs/routes/publish.ipynb 4
async def search_publications(
    auth: dmda.DomoAuth,
    search_term: str = None,
    limit=100,
    offset=0,
    session: httpx.AsyncClient = None,
    debug_api: bool = False,
) -> rgd.ResponseGetData:
    url = f"https://{auth.domo_instance}.domo.com/api/publish/v2/publication/summaries"

    params = {"limit": limit, "offset": offset}

    if search_term:
        params.update({"searchTerm": search_term})

    res = await gd.get_data(
        auth=auth,
        method="GET",
        url=url,
        params=params,
        session=session,
        debug_api=debug_api,
    )

    return res


# %% ../../nbs/routes/publish.ipynb 6
async def get_publication_by_id(
    auth: dmda.DomoAuth,
    publication_id: str,
    session: httpx.AsyncClient = None,
    debug_api: bool = False,
    timeout=10,
) -> rgd.ResponseGetData:
    url = f"https://{auth.domo_instance}.domo.com/api/publish/v2/publication/{publication_id}"

    res = await gd.get_data(
        auth=auth,
        method="GET",
        url=url,
        session=session,
        timeout=timeout,
        debug_api=debug_api,
    )

    return res


# generate publish body


# %% ../../nbs/routes/publish.ipynb 8
def generate_publish_body(
    url: str,
    sub_domain_ls: [str],
    content_ls: [str],
    name: str,
    description: str,
    unique_id: str,
    is_new: bool,
):
    if not sub_domain_ls:
        sub_domain_ls = []

    if not content_ls:
        content_ls = []

    body = {
        "id": unique_id,
        "name": name,
        "description": description,
        "domain": url,
        "content": content_ls,
        "subscriberDomain": sub_domain_ls,
        "new": str(is_new).lower(),
    }

    return body


# %% ../../nbs/routes/publish.ipynb 10
# Creating publish job for a specific subscriber
async def create_publish_job(
    auth: dmda.DomoAuth,
    body: dict,
    session: httpx.AsyncClient = None,
    debug_api: bool = False,
) -> rgd.ResponseGetData:
    url = f"https://{auth.domo_instance}.domo.com/api/publish/v2/publication"

    if debug_api:
        print(body)
    res = await gd.get_data(
        auth=auth,
        method="POST",
        url=url,
        body=body,
        session=session,
        debug_api=debug_api,
    )

    return res


# %% ../../nbs/routes/publish.ipynb 11
# Updating existing publish job with content
async def udpate_publish_job(
    auth: dmda.DomoAuth,
    publication_id: str,
    body: dict,
    session: httpx.AsyncClient = None,
    debug_api: bool = False,
) -> rgd.ResponseGetData:

    url = f"https://{auth.domo_instance}.domo.com/api/publish/v2/publication/{publication_id}"

    res = await gd.get_data(
        auth=auth,
        method="PUT",
        url=url,
        body=body,
        session=session,
        debug_api=debug_api,
    )
    return res


# # finds all jobs waiting to be accepted within the subscriber


# %% ../../nbs/routes/publish.ipynb 13
async def get_subscription_summaries(
    auth: dmda.DomoAuth, session: httpx.AsyncClient = None, debug_api: bool = False
) -> rgd.ResponseGetData:
    """retrieves a summary of existing subscriptions"""
    url = f"https://{auth.domo_instance}.domo.com/api/publish/v2/subscription/summaries"

    res = await gd.get_data(
        auth=auth, method="GET", url=url, session=session, debug_api=debug_api
    )
    return res


# %% ../../nbs/routes/publish.ipynb 15
async def get_subscription_invititations(
    auth: dmda.DomoAuth, session: httpx.AsyncClient = None, debug_api: bool = False
) -> rgd.ResponseGetData:
    """retrieves a list of subscription invitations"""

    url = f"https://{auth.domo_instance}.domo.com/api/publish/v2/subscription/invites"

    res = await gd.get_data(
        auth=auth, method="GET", url=url, session=session, debug_api=debug_api
    )
    return res


# %% ../../nbs/routes/publish.ipynb 18
async def accept_invite_by_id(
    auth: dmda.DomoAuth,
    subscription_id: str,
    session: httpx.AsyncClient = None,
    debug_api: bool = False,
) -> rgd.ResponseGetData:
    """this takes get_subscription_invites_list into account and accepts - not instant"""

    url = f"https://{auth.domo_instance}.domo.com/api/publish/v2/subscription/{subscription_id}"

    res = await gd.get_data(
        auth=auth, method="POST", url=url, session=session, debug_api=debug_api
    )
    return res


# %% ../../nbs/routes/publish.ipynb 19
async def accept_invite_by_id_v2(
    auth: dmda.DomoAuth,
    publication_id: str,
    owner_id: str,
    session: httpx.AsyncClient = None,
    debug_api: bool = False,
) -> rgd.ResponseGetData:
    """this takes get_subscription_invites_list into account and accepts - not instant"""

    url = f"https://{auth.domo_instance}.domo.com/api/publish/v2/subscriptions/v2"

    body = {
        "publicationId": publication_id,
        "customerId": "",
        "domain": "",
        "groupIds": [],
        "userId": owner_id,
        "userIds": [],
    }

    res = await gd.get_data(
        auth=auth,
        method="POST",
        url=url,
        body=body,
        session=session,
        debug_api=debug_api,
    )
    return res


# %% ../../nbs/routes/publish.ipynb 20
async def refresh_publish_jobs(
    auth: dmda.DomoAuth,
    publish_ids: list,
    session: httpx.AsyncClient = None,
    debug_api: bool = False,
) -> rgd.ResponseGetData:
    """Refreshing list of publish jobs. Typically "instance" = publisher instance"""

    url = f"https://{auth.domo_instance}.domo.com/api/publish/v2/publication/refresh"

    body = {"publicationIds": publish_ids}

    res = await gd.get_data(
        auth=auth,
        method="PUT",
        url=url,
        body=body,
        session=session,
        debug_api=debug_api,
    )
    return res



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/classes/50_DomoGroup_Membership.ipynb.

# %% ../../nbs/classes/50_DomoGroup_Membership.ipynb 2
from __future__ import annotations
from typing import List

import httpx

from nbdev.showdoc import patch_to

import domolibrary.utils.chunk_execution as ce

import domolibrary.client.DomoAuth as dmda
import domolibrary.client.DomoError as de

import domolibrary.routes.group as group_routes

import domolibrary.classes.DomoUser as dmu

# %% auto 0
__all__ = ["UpdateGroupMembership", "GroupMembership"]


# %% ../../nbs/classes/50_DomoGroup_Membership.ipynb 4
class UpdateGroupMembership(de.DomoError):
    def __init__(self, member_name, group_name, domo_instance):
        super().__init__(
            domo_instance=domo_instance,
            message=f"unable to add {member_name} to {group_name}",
        )


class GroupMembership:
    _add_member_ls: list[str]
    _remove_group_ls: list[str]

    _add_owners_ls: list[str]
    _remove_owner_ls: list[str]

    _current_member_ls: list[str]
    _current_owner_ls: list[str]

    group = None

    def __init__(self, group):
        self.group = group

        self._add_member_ls: List[dict] = []
        self._remove_member_ls: List[dict] = []

        self._add_owner_ls: List[dict] = []
        self._remove_owner_ls: List[dict] = []

        self._current_member_ls = []
        self._current_owner_ls = []

    def _add_to_list(self, member, list_to_update, debug_prn: bool = False):
        import domolibrary.classes.DomoUser as dmu
        import domolibrary.classes.DomoGroup as dmg

        match_obj = next(
            (
                user_obj
                for user_obj in list_to_update
                if user_obj.get("id") == member.id
            ),
            None,
        )
        if match_obj:
            print(f"➡️ {member}  of type {type(member).__name__} already in ls")
            return list_to_update

        if debug_prn:
            print(
                f"➡️ adding {member.id}  of type {type(member).__name__} to {self.group.name}"
            )

        if isinstance(member, dmu.DomoUser):
            list_to_update.append({"id": str(member.id), "type": "USER"})

            return list_to_update

        if isinstance(member, dmg.DomoGroup):
            list_to_update.append({"id": str(member.id), "type": "GROUP"})

            return list_to_update

        member_name = (
            getattr(member, "name", None)
            or getattr(member, "display_name", None)
            or "name not provided"
        )

        raise UpdateGroupMembership(
            domo_instance=self.group.auth.domo_instance,
            group_name=self.group.name,
            member_name=member_name,
        )

    def _add_member(self, member, debug_prn: bool = False):
        return self._add_to_list(member, self._add_member_ls, debug_prn)

    def _remove_member(self, member, debug_prn: bool = False):
        if type(member).__name__ == "DomoGroup" and member.type == "system":
            if debug_prn:
                print(f"remove_owner - skipping {member.name} type is {member.type}")
            return
        return self._add_to_list(member, self._remove_member_ls, debug_prn)

    def _add_owner(self, member, debug_prn: bool = False):
        return self._add_to_list(member, self._add_owner_ls, debug_prn)

    def _remove_owner(self, member, debug_prn: bool = False):
        if type(member).__name__ == "DomoGroup" and member.type == "system":
            if debug_prn:
                print(f"remove_owner - skipping {member.name} type is {member.type}")
            return

        return self._add_to_list(member, self._remove_owner_ls, debug_prn)

    def _reset_obj(self):
        self._add_member_ls = []
        self._remove_member_ls = []

        self._add_owner_ls = []
        self._remove_owner_ls = []

    async def _update_group_access(
        self,
        debug_api: bool = False,
        session: httpx.AsyncClient = None,
    ):
        res = await group_routes.update_group_membership(
            auth=self.group.auth,
            group_id=self.group.id,
            add_member_arr=self._add_member_ls,
            remove_member_arr=self._remove_member_ls,
            add_owner_arr=self._add_owner_ls,
            remove_owner_arr=self._remove_owner_ls,
            debug_api=debug_api,
            session=session,
        )
        self._reset_obj()

        # add
        # remove
        # set


# %% ../../nbs/classes/50_DomoGroup_Membership.ipynb 6
@patch_to(GroupMembership)
async def get_owners(
    self: GroupMembership,
    auth: dmda.DomoAuth = None,
    return_raw: bool = False,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
):
    import domolibrary.classes.DomoUser as dmu
    import domolibrary.classes.DomoGroup as dmg

    auth = auth or self.group.auth

    self._current_owner_ls = []

    res = await group_routes.get_group_owners(
        group_id=self.group.id, auth=self.group.auth
    )
    if return_raw:
        return res

    group_ids = [obj.get("id") for obj in res.response if obj.get("type") == "GROUP"]
    if group_ids:
        domo_groups = await ce.gather_with_concurrency(
            n=60,
            *[
                dmg.DomoGroup.get_by_id(group_id=group_id, auth=auth)
                for group_id in group_ids
            ],
        )
        self._current_owner_ls += domo_groups

    user_ids = [obj.get("id") for obj in res.response if obj.get("type") == "USER"]
    if user_ids:
        domo_users = await dmu.DomoUsers.by_id(
            user_ids=user_ids, auth=auth, only_allow_one=False
        )
        self._current_owner_ls += domo_users

    self.group.owner_id_ls = group_ids + user_ids
    self.group.owner_ls = self._current_owner_ls

    return self._current_owner_ls
    # return domo_users


# %% ../../nbs/classes/50_DomoGroup_Membership.ipynb 7
@patch_to(GroupMembership)
async def get_members(
    self: GroupMembership,
    auth: dmda.DomoAuth = None,
    return_raw: bool = False,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
):
    import domolibrary.classes.DomoUser as dmu

    auth = auth or self.group.auth

    self._current_member_ls = []

    res = await group_routes.get_group_membership(
        group_id=self.group.id, auth=self.group.auth
    )
    if return_raw:
        return res

    user_ids = [obj.get("userId") for obj in res.response]
    if user_ids:
        domo_users = await dmu.DomoUsers.by_id(
            user_ids=user_ids, auth=auth, only_allow_one=False
        )
        self._current_member_ls += domo_users

    self.group.members_id_ls = user_ids
    self.group.members_ls = self._current_member_ls

    return self.group.members_ls


# %% ../../nbs/classes/50_DomoGroup_Membership.ipynb 9
@patch_to(GroupMembership)
async def add_members(
    self: GroupMembership,
    add_user_ls: list[dmu.DomoUser],
    return_raw: bool = False,
    debug_api: bool = False,
    debug_prn: bool = False,
    session: httpx.AsyncClient = None,
):
    self._reset_obj()

    [self._add_member(domo_user, debug_prn) for domo_user in add_user_ls]

    res = await self._update_group_access(debug_api=debug_api, session=session)

    if return_raw:
        return res

    return await self.get_members()


@patch_to(GroupMembership)
async def remove_members(
    self: GroupMembership,
    remove_user_ls: list[dmu.DomoUser],
    return_raw: bool = False,
    debug_api: bool = False,
    debug_prn: bool = False,
    session: httpx.AsyncClient = None,
):
    self._reset_obj()

    [self._remove_member(domo_user, debug_prn) for domo_user in remove_user_ls]

    res = await self._update_group_access(debug_api=debug_api, session=session)

    if return_raw:
        return res

    return await self.get_members()


@patch_to(GroupMembership)
async def set_members(
    self: GroupMembership,
    user_ls: list[dmu.DomoUser],
    return_raw: bool = False,
    debug_api: bool = False,
    debug_prn: bool = False,
    session: httpx.AsyncClient = None,
):
    self._reset_obj()

    domo_users = await self.get_members()

    if debug_prn:
        print({"domo_users": domo_users, "user_ls": user_ls})

    [self._add_member(domo_user, debug_prn) for domo_user in user_ls]
    [
        self._remove_member(domo_user, debug_prn)
        for domo_user in domo_users
        if domo_user not in user_ls
    ]

    res = await self._update_group_access(debug_api=debug_api, session=session)

    if return_raw:
        return res

    return await self.get_members()


# %% ../../nbs/classes/50_DomoGroup_Membership.ipynb 10
@patch_to(GroupMembership)
async def add_owners(
    self: GroupMembership,
    add_owner_ls,
    return_raw: bool = False,
    debug_api: bool = False,
    debug_prn: bool = False,
    session: httpx.AsyncClient = None,
):
    self._reset_obj()

    [self._add_owner(domo_user, debug_prn) for domo_user in add_owner_ls]

    res = await self._update_group_access(debug_api=debug_api, session=session)

    if return_raw:
        return res

    return await self.get_owners()


@patch_to(GroupMembership)
async def remove_owners(
    self: GroupMembership,
    remove_owner_ls,
    return_raw: bool = False,
    debug_api: bool = False,
    debug_prn: bool = False,
    session: httpx.AsyncClient = None,
):
    self._reset_obj()

    [self._remove_owner(domo_user, debug_prn) for domo_user in remove_owner_ls]

    res = await self._update_group_access(debug_api=debug_api, session=session)

    if return_raw:
        return res

    return await self.get_owners()


@patch_to(GroupMembership)
async def set_owners(
    self: GroupMembership,
    owner_ls,
    return_raw: bool = False,
    debug_api: bool = False,
    debug_prn: bool = False,
    session: httpx.AsyncClient = None,
):
    self._reset_obj()

    domo_users = await self.get_owners()

    if debug_prn:
        print({"domo_users": domo_users, "user_ls": owner_ls})

    [self._add_owner(domo_user, debug_prn) for domo_user in owner_ls]
    [
        self._remove_owner(domo_user, debug_prn)
        for domo_user in domo_users
        if domo_user not in owner_ls
    ]

    res = await self._update_group_access(debug_api=debug_api, session=session)

    if return_raw:
        return res

    return await self.get_owners()


# %% ../../nbs/classes/50_DomoGroup_Membership.ipynb 11
@patch_to(GroupMembership)
async def add_owner_manage_groups_role(self: GroupMembership):
    import domolibrary.classes.DomoGroup as dmg

    await dmg.DomoGroups.toggle_system_group_visibility(
        auth=self.group.auth, is_hide_system_groups=False
    )

    grant_group = await dmg.DomoGroup.search_by_name(
        auth=self.group.auth, group_name="Grant: Manage all groups"
    )

    await self.add_owners(add_owner_ls=[grant_group])

    await dmg.DomoGroups.toggle_system_group_visibility(
        auth=self.group.auth, is_hide_system_groups=True
    )



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/classes/50_DomoPDP.ipynb.

# %% auto 0
__all__ = ["PDP_Parameter", "PDP_Policy", "Dataset_PDP_Policies", "SearchPDP_NotFound"]

# %% ../../nbs/classes/50_DomoPDP.ipynb 2
import httpx

from nbdev.showdoc import patch_to

from dataclasses import dataclass

import domolibrary.utils.DictDot as util_dd
import domolibrary.utils.chunk_execution as ce

import domolibrary.routes.pdp as pdp_routes

import domolibrary.client.DomoAuth as dmda
import domolibrary.client.DomoError as de


# %% ../../nbs/classes/50_DomoPDP.ipynb 6
@dataclass
class PDP_Parameter:
    column_name: str
    column_values_ls: list
    operator: str = (
        "EQUALS"
        or "GREATER_THAN"
        or "LESS_THAN"
        or "GREATER_THAN_EQUAL"
        or "LESS_THAN_EQUAL"
        or "BETWEEN"
    )
    ignore_case: bool = True
    type: str = (
        "COLUMN" or "DYNAMIC"
    )  # column sets parameter on data vs dynamic creates on Domo Trusted Attribute


# %% ../../nbs/classes/50_DomoPDP.ipynb 7
@patch_to(PDP_Parameter)
def generate_parameter_simple(obj):

    return pdp_routes.generate_policy_parameter_simple(
        column_name=obj.name,
        type=obj.type,
        column_values_ls=obj.values,
        operator=obj.operator,
        ignore_case=obj.ignoreCase,
    )


@patch_to(PDP_Parameter)
def generate_body_from_parameter(self):

    return pdp_routes.generate_policy_parameter_simple(
        column_name=self.column_name,
        type=self.type,
        column_values_ls=self.column_values_ls,
        operator=self.operator,
        ignore_case=self.ignore_case,
    )


# %% ../../nbs/classes/50_DomoPDP.ipynb 10
@dataclass
class PDP_Policy:
    dataset_id: str
    filter_group_id: str
    name: str
    # resources: list
    parameters_ls: list[dict]
    user_ls: list[str]
    group_ls: list[str]
    virtual_user_ls: list[str]

    @classmethod
    async def _from_json(cls, obj, auth: dmda.DomoAuth):
        dd = util_dd.DictDot(obj)

        import domolibrary.classes.DomoUser as dmu
        import domolibrary.classes.DomoGroup as dmg

        return cls(
            dataset_id=dd.dataSourceId,
            filter_group_id=dd.filterGroupId,
            name=dd.name,
            # resources=dd.resources,
            parameters_ls=dd.parameters,
            user_ls=(
                await ce.gather_with_concurrency(
                    n=60,
                    *[
                        dmu.DomoUser.get_by_id(user_id=id, auth=auth)
                        for id in dd.userIds
                    ],
                )
                if dd.userIds
                else None
            ),
            group_ls=(
                await ce.gather_with_concurrency(
                    n=60,
                    *[
                        dmg.DomoGroup.get_by_id(group_id=id, auth=auth)
                        for id in dd.groupIds
                    ],
                )
                if dd.groupIds
                else None
            ),
            virtual_user_ls=dd.virtualUserIds,
        )

    @classmethod
    async def upsert_policy(
        cls,
        auth: dmda.DomoAuth,
        dataset_id: str,
        # body sent to the API (uses camelCase instead of snake_case)
        policy_definition: dict,
        debug_api: bool = False,
        debug_prn: bool = False,
    ):

        # print(policy_definition)
        policy_id = policy_definition.get("filterGroupId")
        if policy_id:
            if debug_prn:
                print(f"Updating policy {policy_id} in {auth.domo_instance}")
            res = await pdp_routes.update_policy(
                auth=auth,
                dataset_id=dataset_id,
                policy_id=policy_id,
                body=policy_definition,
                debug_api=debug_api,
            )
            return res
        else:
            if debug_prn:
                print(f"Policy does not exist. Creating policy in {auth.domo_instance}")
            res = await pdp_routes.create_policy(
                auth=auth,
                dataset_id=dataset_id,
                body=policy_definition,
                debug_api=debug_api,
            )
            return res


# %% ../../nbs/classes/50_DomoPDP.ipynb 12
@patch_to(PDP_Policy)
def generate_body_from_policy(
    self: PDP_Policy,
    # params: list[dict] = ''
):
    if not self.parameters_ls:
        raise Exception("generate_body_from_policy: no parameters")

    self.parameters_ls = [
        PDP_Parameter.generate_parameter_simple(param) for param in self.parameters_ls
    ]

    return pdp_routes.generate_policy_body(
        policy_name=self.name,
        dataset_id=self.dataset_id,
        parameters_ls=self.parameters_ls,
        policy_id=self.filter_group_id,
        user_ids=self.user_ls,
        group_ids=self.group_ls,
        virtual_user_ids=self.virtual_user_ls,
    )


# %% ../../nbs/classes/50_DomoPDP.ipynb 14
class Dataset_PDP_Policies:

    dataset = None  # domo dataset class
    policies: list[PDP_Policy] = None
    auth = None

    def __init__(self, dataset):
        self.dataset = dataset
        self.policies = []


# %% ../../nbs/classes/50_DomoPDP.ipynb 16
@patch_to(Dataset_PDP_Policies)
async def get_policies(
    self: Dataset_PDP_Policies,
    include_all_rows: bool = True,
    auth: dmda.DomoAuth = None,
    dataset_id: str = None,
    return_raw: bool = False,
    debug_api: bool = False,
):

    dataset_id = dataset_id or self.dataset.id
    auth = auth or self.dataset.auth

    res = await pdp_routes.get_pdp_policies(
        auth=auth,
        dataset_id=dataset_id,
        debug_api=debug_api,
        include_all_rows=include_all_rows,
    )

    if return_raw:
        return res

    if res.status == 200:
        domo_policy = await ce.gather_with_concurrency(
            n=60,
            *[
                PDP_Policy._from_json(policy_obj, auth=auth)
                for policy_obj in res.response
            ],
        )
        self.policies = domo_policy
        return domo_policy


# %% ../../nbs/classes/50_DomoPDP.ipynb 25
class SearchPDP_NotFound(de.DomoError):
    def __init__(
        self, domo_instance, dataset_id, message="not found", function_name="search_pdp"
    ):

        super().__init__(
            domo_instance=domo_instance,
            entity_id=dataset_id,
            message=message,
            function_name=function_name,
        )


@patch_to(Dataset_PDP_Policies, cls_method=True)
async def search_pdp_policies(
    cls: Dataset_PDP_Policies,
    auth: dmda.DomoAuth,
    search: str,
    dataset_id: str = None,
    search_method: str = "id" or "name",
    is_exact_match: bool = True,
    return_raw: bool = False,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
):

    all_pdp_policies = await Dataset_PDP_Policies(cls).get_policies(
        auth=auth, dataset_id=dataset_id, debug_api=debug_api
    )

    if return_raw:
        return all_pdp_policies

    if search_method == "name":
        if is_exact_match:
            policy_search = next(
                (policy for policy in all_pdp_policies if policy.name == search), None
            )
            # print(policy_search)

            if not policy_search:
                raise SearchPDP_NotFound(
                    dataset_id=dataset_id,
                    message=f'There is no policy named "{search}" on dataset_id {dataset_id}',
                    domo_instance=auth.domo_instance,
                )

            return policy_search
        else:
            policy_search = [
                policy
                for policy in all_pdp_policies
                if search.lower() in policy.name.lower()
            ]
            if not policy_search:
                raise SearchPDP_NotFound(
                    dataset_id=dataset_id,
                    message=f'There is no policy name containing "{search}" on dataset_id {dataset_id}',
                    domo_instance=auth.domo_instance,
                )

            return policy_search
    else:
        policy_search = next(
            (policy for policy in all_pdp_policies if policy.filter_group_id == search),
            None,
        )

    if not policy_search:
        raise SearchPDP_NotFound(
            dataset_id=dataset_id,
            message=f'There is no policy id "{search}" on dataset_id {dataset_id}',
            domo_instance=auth.domo_instance,
        )

    return policy_search


# %% ../../nbs/classes/50_DomoPDP.ipynb 29
@patch_to(PDP_Policy)
async def delete_policy(
    self: PDP_Policy,
    auth: dmda.DomoAuth,
    policy_id: str = None,
    dataset_id: str = None,
    debug_api: bool = False,
):

    dataset_id = dataset_id or self.dataset_id
    policy_id = policy_id or self.filter_group_id

    res = await pdp_routes.delete_policy(
        auth=auth, dataset_id=dataset_id, policy_id=policy_id, debug_api=debug_api
    )

    return res


# %% ../../nbs/classes/50_DomoPDP.ipynb 33
@patch_to(Dataset_PDP_Policies)
async def toggle_dataset_pdp(
    self: Dataset_PDP_Policies,
    auth: dmda.DomoAuth = None,
    dataset_id: str = None,
    is_enable: bool = True,  # True will enable pdp, False will disable pdp
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
):
    auth = auth or self.dataset.auth

    return await pdp_routes.toggle_pdp(
        auth=auth,
        dataset_id=dataset_id or self.dataset.id,
        is_enable=is_enable,
        debug_api=debug_api,
        session=session,
    )



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/Image.ipynb.

# %% auto 0
__all__ = [
    "domo_default_img",
    "isBase64",
    "handle_string_to_bytes_and_decode",
    "handle_string_to_bytes_and_encode",
    "to_bytes",
    "crop_square",
    "are_same_image",
]

# %% ../../nbs/utils/Image.ipynb 2
from PIL.Image import Image

# %% ../../nbs/utils/Image.ipynb 3
from typing import Union
import types
import PIL
import numpy as np

import os
import io
import base64

from nbdev.showdoc import patch_to


# %% ../../nbs/utils/Image.ipynb 4
def isBase64(s):
    try:
        return base64.b64encode(base64.b64decode(s)) == s
    except Exception:
        return False


def handle_string_to_bytes_and_decode(data: Union[str, bytes]):

    if isinstance(data, str):
        data = bytes(data)

    if isBase64(data):
        data = base64.b64decode(data)

    return data


def handle_string_to_bytes_and_encode(data: Union[str, bytes]):

    if isinstance(data, str):
        data = bytes(data)

    if not isBase64(data):
        data = base64.b64encode(data)

    return data


# %% ../../nbs/utils/Image.ipynb 5
def to_bytes(self) -> bytes:
    byte_arr = io.BytesIO()

    if not hasattr(self, "area"):
        self.area = self

    self.area.save(byte_arr, format=self.format)

    self.data = byte_arr.getvalue()

    return self.data


def crop_square(self):

    width, height = self.size  # Get dimensions

    new_edge = min(width, height)

    left = (width - new_edge) / 2
    top = (height - new_edge) / 2
    right = (width + new_edge) / 2
    bottom = (height + new_edge) / 2

    # Crop the center of the image
    self.area = self.crop((left, top, right, bottom))

    self.to_bytes()

    return self.area


@patch_to(Image, cls_method=True)
def from_image_file(cls, image_path: str) -> Image:
    if not os.path.exists(image_path):
        raise FileNotFoundError(image_path)

    with open(image_path, "rb") as file:
        data = file.read()

    data = handle_string_to_bytes_and_decode(data)

    im = PIL.Image.open(io.BytesIO(data))

    im.to_bytes = types.MethodType(to_bytes, im)
    im.crop_square = types.MethodType(crop_square, im)

    return im


# %% ../../nbs/utils/Image.ipynb 8
@patch_to(Image, cls_method=True)
def from_bytestr(cls, data: Union[str, bytes]) -> Image:

    data = handle_string_to_bytes_and_decode(data)

    im = PIL.Image.open(io.BytesIO(data))

    im.to_bytes = types.MethodType(to_bytes, im)
    im.crop_square = types.MethodType(crop_square, im)

    return im


# %% ../../nbs/utils/Image.ipynb 11
default_img_bytes = b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x01,\x00\x00\x01,\x08\x06\x00\x00\x00y}\x8eu\x00\x00\x0eUIDATx\x9c\xed\xdd\xe9r\xda\xd8\x16@\xe1-\x81\x84\x18M\x02\xb1\x93T\xbf\xff\xa3\xb9\xc1\x063\nM\xe8\xe8\xfe\xe86\x95\xdc\x0c\x9d86\xe7\xec\xa3\xf5UQvuW%;\x12Z\x80&\x82\xcdf\xd3\x08\x00(\x10\xda\x1e\x00\x00~\x15\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0\x06\xc1\x02\xa0F\xd7\xf6\x00\xd0\xabi\x1a1\xc6\x881F\x9a\xa6\x91\xa6iDD\xc4\x18#""a\xf8\xcf\xeba\x10\x04\x12\x04\x81\x84a(a\x18J\x10\x04\xd6f\x86n\x04\x0b\xbf\xa4\xaek\xc9\xf3\\\xf2<\x97\xaa\xaa\xa4,K\xa9\xaa\xeaE\x7fV\x14E\x12\xc7\xb1DQ$I\x92H\x92$\xd2\xe9t^yb\xf8\x88`\xe1\x87\xb2,\x93\xd3\xe9$Y\x96IQ\x14\xaf\xf6\xe7VU\xf5M\xecz\xbd\x9e\xf4\xfb}\x19\x0c\x06\xd2\xef\xf7_\xed\xef\x82_\x82\xcdf\xd3\xd8\x1e\x02\xee\xa8\xaaJ\x0e\x87\x83\x1c\x8f\xc7\x17\xbf\x83\xfaSQ\x14\xc9h4\x92\xf1x,Q\x14Y\x99\x01n"X\x10\x11\x914Me\xbb\xddJ\x9e\xe7\xb6G\xf9J\x92$2\x9dNe8\x1c\xda\x1e\x05\x0e X-w8\x1cd\xbb\xddJY\x96\xb6G\xf9\xa98\x8ee:\x9d\xcax<\xb6=\n,"X-\x95\xa6\xa9\xac\xd7kk\x1f\xfb^*\x8a"\x99\xcdf\xbc\xe3j)\x82\xd52\xe7\xf3Y\x1e\x1f\x1f\xe5t:\xd9\x1e\xe5\x8f\x0c\x06\x03\xf9\xf0\xe1\x83t\xbb\x1c7j\x13\x82\xd5\x12M\xd3\xc8f\xb3\x91\xedv{9_J\xbb \x08d:\x9d\xca\xbbw\xef8\xb7\xab%\x08V\x0bTU%\xcb\xe5\xf2UOMpI\xaf\xd7\x93\xbb\xbb;\x8e(\xb6\x00\xc1\xf2\\\x9a\xa6\xf2\xf0\xf0p9\xfb\xdcWa\x18\xca\xed\xed-\xfb\xb6<G\xb0<\xb6^\xafe\xbb\xdd\xda\x1e\xe3\xaa\xa6\xd3\xa9\xccf3\xdbc\xe0\x8d\xb0\xc7\xd2CM\xd3\xc8\xc3\xc3\x83\x1c\x8fG\xdb\xa3\\\xddv\xbb\x95\xf3\xf9,\xb7\xb7\xb7\xec\xd7\xf2\x10\xc1\xf2\x8c1F\x16\x8b\x85dYf{\x14k\x8e\xc7\xa3\xd4u-\x1f?~\xbc\\\x80\r?\xb06=b\x8c\x91\xfb\xfb\xfbV\xc7\xeaY\x96er\x7f\x7f\xef\xfd\xbe\xbb\xb6!X\x9ex\x8e\x95\xafG\x02_\xa2(\n\xa2\xe5\x19\x82\xe5\x81\xa6i\xbc>m\xe1O\x14E!\xcb\xe5\xd2\x9bs\xcf\xda\x8e`y\xc0\x873\xd7\xdf\xd2\xe9t\x92\xc7\xc7G\xdbc\xe0\x15\x10,\xe5\x9e\x9e\x9e\xe4p8\xd8\x1e\xc3y\x87\xc3A\x9e\x9e\x9el\x8f\x81?D\xb0\x14\xcb\xf3\\6\x9b\x8d\xed1\xd4\xd8l6\xce\xdd>\x07\xbf\x87`)U\xd7\xb5,\x16\x0b\xdbc\xa8\xb3X,\xa4\xaek\xdbc\xe0\x85\x08\x96R\xcb\xe5\x92\r\xef\x05\xea\xba\x96\xe5ri{\x0c\xbc\x10\xc1Rh\xbf\xdfs\xae\xd5\x1f\xc8\xb2L\xf6\xfb\xbd\xed1\xf0\x02\x04K\x19c\x8c\xac\xd7k\xdbc\xa8\xb7^\xaf9?K!\x82\xa5\xccj\xb5bC{\x05\xc6\x18Y\xadV\xb6\xc7\xc0o"X\x8a\xe4y\xce)\x0c\xaf\xe8p8p\xd4P\x19\x82\xa5\x08\x1f\x05_\x1f\xcbT\x17\x82\xa5\xc4\xf3\xb7.\xe3u\xb1\\u!XJp\x82\xe8\xdba\xd9\xeaA\xb0\x14(\x8a\x82k\x05\xdf\xd0\xe9t\xe2\xc2q%\x08\x96\x02\xbb\xdd\xce\xf6\x08\xdec\x19\xeb@\xb0\x1cg\x8c\x914Mm\x8f\xe1\xbd4M9]D\x01\x82\xe586\xa4\xeb\xe0\x85A\x07\x82\xe58\xce\xbb\xba\x1e\x96\xb5\xfb\x08\x96\xc3\xea\xba\xe6\x9a\xc1+\xca\xb2\x8c\x0b\xca\x1dG\xb0\x1cF\xac\xae\x8fe\xee6\x82\xe506\x9e\xebc\x99\xbb\x8d`9\x8cs\xaf\xae\x8fe\xee6\x82\xe5\xa8\xaa\xaa\xe4|>\xdb\x1e\xa3u\xce\xe7\xb3TUe{\x0c\xfc\x00\xc1r\x14g^\xdb\xc3\xb2w\x17\xc1r\x14\xaf\xf2\xf6\xb0\xec\xddE\xb0\x1c\xc5Fc\x0f\xcb\xde]\x04\xcbQeY\xda\x1e\xa1\xb5X\xf6\xee"X\x8e\xe2U\xde\x1e\x96\xbd\xbb\x08\x96\x83\x9a\xa6\xe1\xfaA\x8b\x8c1\xd24\x8d\xed1\xf0\x1d\x04\xcbA\xc4\xca>\xd6\x81\x9b\x08\x96\x83xu\xb7\x8fu\xe0&\x82\xe5 .\xc0\xb5\x8fu\xe0&\x82\xe5 ^\xdd\xedc\x1d\xb8\x89`9(\x08\x02\xdb#\xb4\x1e\xeb\xc0M\x04\xcbAl,\xf6\xb1\x0e\xdcD\xb0\x1c\x14\x86\xac\x16\xdbX\x07nb\xad8\x88Ww\xfbX\x07n"X\x0e\xeat:\xb6Gh=\xd6\x81\x9b\x08\x96\xa3\xba\xdd\xae\xed\x11Z\x8be\xef.\x82\xe5\xa88\x8em\x8f\xd0Z,{w\x11,G\xb1\xd1\xd8\xc3\xb2w\x17\xc1rT\x14E\xb6Gh-\x96\xbd\xbb\x08\x96\xa3\xd8h\xeca\xd9\xbb\x8b`9*I\x12\x0e\xad[\x10\x04\x81$Ib{\x0c\xfc\x00\xc1r\x14\x1b\x8e\x1d\xbcP\xb8\x8d`9\xac\xdf\xef\xdb\x1e\xa1uX\xe6n#X\x0ec\xe3\xb9>\x96\xb9\xdb\x08\x96\xc3\x92$\xe1$\xc6+\xeav\xbb|\x0cw\x1c\xc1r\xdch4\xb2=Bk\xb0\xac\xddG\xb0\x1c7\x1e\x8fm\x8f\xd0\x1a,k\xf7\x11,\xc7\xc5q\xcc\x99\xd7W\xc0r\xd6\x81`)0\x99Ll\x8f\xe0=\x96\xb1\x0e\x04K\x81\xc9d\xc2\xedN\xdeP\xa7\xd3!XJ\x10,\x05\x82 \x90\xe9tj{\x0coM\xa7SN\x16U\x82`)1\x99L\xb8m\xef\x1b\x08\xc3\x90wW\x8a\xb0\x05(\x11\x86\xa1\xdc\xdc\xdc\xd8\x1e\xc3;777\xbc\x10(\xc2\x9aRd:\x9dr\'\x81W\x14E\x11\x1f\xb5\x95!X\x8a\x84a(\xb3\xd9\xcc\xf6\x18\xde\x98\xcdf\xbc\xbbR\x86\xb5\xa5\xccp8\x94\xe1ph{\x0c\xf5X\x8e:\x11,\x85\xe6\xf39\xef\x0c\xfe@\x18\x862\x9f\xcfm\x8f\x81\x17\xe0Y\xafP\xb7\xdbe\x83\xfb\x03\xf3\xf9\x9c\x8b\xca\x95"XJ\x8d\xc7c\xae}{\x01\x96\x9bn\x04K\xb1\xf9|\xce\xf5o\xbf!\x8ec\xde\x99*G\xb0\x14\x0b\xc3P\xee\xee\xee\xd8\x9f\xf5\x0bXV~`\xed)\x17\xc7\xb1|\xfa\xf4\x89KK~"\x08\x02\xf9\xf4\xe9\x13\xefF=@\xb0<\x90$\x89\xdc\xdd\xdd\xd9\x1e\xc3Ywww\xdcI\xd4\x13\x04\xcb\x13\xc3\xe1Pnoom\x8f\xe1\x9c\xdb\xdb[\xce\xb7\xf2\x08\xc1\xf2\xc8x<&Z_\xb8\xbd\xbd\xe5\x88\xa0g8\x19\xc53\xe3\xf1X:\x9d\x8e,\x97K1\xc6\xd8\x1e\xc7\x8a\xe7\x1d\xec\x83\xc1\xc0\xf6(xe\xbc\xc3\xf2\xd0`0\x90\xcf\x9f?\xb7\xf2\x88X\x18\x86\xf2\xf9\xf3gb\xe5\xa9\xf6=\xa3[\xa2\xd7\xeb\xc9_\x7f\xfd%\xbd^\xcf\xf6(W\xd3\xc6\x7fs\xdb\x04\x9b\xcd\xa6\xb1=\x04\xdeN\xd34\xb2^\xafe\xb7\xdb\xd9\x1e\xe5M\xdd\xdc\xdc\xc8l6\xe3\xf4\x0e\xcf\x11\xac\x96H\xd3T\x1e\x1e\x1e\xbc\xdb\xaf\x15\x86!G\x02[\x84`\xb5\x881F\xd6\xeb\xb5\xec\xf7{\xdb\xa3\xbc\x8a\xc9d\xc2=\xadZ\x86`\xb5P\x9e\xe7\xf2\xf8\xf8(eY\xda\x1e\xe5E\xe28\x96\x0f\x1f>p2h\x0b\x11\xac\x16\xdb\xef\xf7\xb2\xddn\xa5\xaa*\xdb\xa3\xfc\x92\xe7[\x1a\xf3\xa5\x11\xedE\xb0 \xc7\xe3Q\xb6\xdb\xad\x14Ea{\x94\xef\xea\xf5z2\x9dNe4\x1a\xd9\x1e\x05\x96\x11,\\\x9cN\'9\x1c\x0e\x92\xa6\xa94\x8d\xdd\xa7E\x10\x042\x1c\x0ee<\x1esN\x15.\x08\x16\xbea\x8c\x914Me\xbf\xdfK\x9e\xe7W\xfd\xbb\x93$\x91\xc9d"\xc3\xe1\x90\x9d\xe9\xf8\x06\xc1\xc2O\x19c$\xcfs9\x9dN\x92e\xd9\xab\xef\xa8\x8f\xe3X\xfa\xfd\xbe\x0c\x06\x03I\x92\x84H\xe1\xa7\x08\x16~\x8b1F\xaa\xaa\x92\xb2,\xe5|>_~\x1ac.\x8f\xe7\x8f\x93A\x10H\x18\x86\x97G\xb7\xdb\x958\x8e/?\xa3("P\xf8-\\\xfc\x8c\xdf\x12\x86\xa1\xf4z=.\x7f\x81\x15\xbc\xbc\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83`\x01P\x83\xef%\xc4\x0f5M#u]\xcb\xf9|\x96\xba\xae\xa5\xaek1\xc6|\xf5{\xd34b\x8c\x11\x11\xf9\xe6\xe7\xf3\x97\xa4~\xf9\xf3\xf9\xcbU;\x9d\x8et:\x9d\xaf~\xefv\xbb\xd2\xe9t$\x08\x02\x0b\xffZh@\xb0Z\xee\xcboo~~TUu\xf9\xdd\x86n\xb7+\xddnW\xa2(\xba\xfc\xfe\xe5\xb7F\xa3\xbdX\xfb-q>\x9f\xa5(\x8a\xcb\xd7\xcc??\x9e\xbfV\xde%\xcf\xb1\xcc\xf3\xfc\x9b\xff\x17\x04\x81\xc4q|yDQ$\xbd^\x8f\x90\xb5\x04k\xd9CUUI\x9e\xe7R\x96\xa5\xe4y.EQ8\x19\xa6\x97h\x9aF\x8a\xa2\x90\xa2(\xbe\xfa\xefA\x10H\xaf\xd7\x93$I$\x8ecI\x92D\xa2(\xb24%\xde\n\xc1R\xce\x18#Y\x96IQ\x14\x978=\xefCj\x93\xa6i$\xcf\xf3\xaf\xde\x95\x85ax\x89X\xaf\xd7\x93~\xbf\x7f\xd9\x9f\x06\x9d\x82\xcdf\xe3\xc7KoK\xd4u-y\x9eK\x96e\x97@\xe1\xd7=\x07\xac\xdf\xefK\x92$\xd2\xe9tl\x8f\x84\xdf@\xb0\x1c\xd74\x8ddYvy\x10\xa8\xd7\xf5\xfc\xce\xeb\xf9\xc1\x11J\xb7\x11,\x07\xd5u-\xa7\xd3I\xd24\x95\xd3\xe9\xe4\xcd\xfe\'\xd7\x05A \x83\xc1@\x86\xc3\xa1\x0c\x06\x03\xde}9\x88`9\xa2,\xcbK\xa4\xbewt\x0c\xd7\x97$\xc9%^q\x1c\xdb\x1e\x07B\xb0\xac\xaa\xaaJ\x8e\xc7\xa3\x1c\x0e\x07\xa9\xaa\xca\xf68\xf8\x89(\x8ad<\x1e\xcbh4\xe2\xe8\xa3E\x04\xeb\xca\x8c1\x97H\xf1NJ\xa7$I.\xf1\xe2\xa8\xe3u\x11\xac+h\x9aFN\xa7\x93\x1c\x0e\x07\xf6Iy\xe4y\x9f\xd7x<\x96\xc1`\xc0\x0e\xfb+ Xo\xe8|>\xcb~\xbf\x97\xfd~/u]\xdb\x1e\x07o\xa8\xd3\xe9\xc8d2\x91\xc9d\xc2Y\xf7o\x88`\xbd\x81\xa2(d\xb7\xdb\xc9\xf1x\xe4\xddT\xcb\x04A \xa3\xd1Hnnn\xa4\xd7\xeb\xd9\x1e\xc7;\x04\xeb\x15\xa5i*\xdb\xed\x96}S\x10\x91\x7f\xf6uM\xa7S\x19\x0e\x87\xb6G\xf1\x06\xc1z\x05\x87\xc3A6\x9b\rG\xfa\xf0]Q\x14\xc9\xbbw\xefd<\x1e\xdb\x1eE=\x82\xf5\x07\x8e\xc7\xa3<==\x11*\xfc\x92(\x8a\xe4\xfd\xfb\xf72\x1a\x8dl\x8f\xa2\x16\xc1z\x81\xd3\xe9$\xeb\xf5Z\xca\xb2\xb4=\n\x14\x8a\xe3Xf\xb3\x99\x0c\x06\x03\xdb\xa3\xa8C\xb0~C\x9e\xe7\xf2\xf4\xf4$Y\x96\xd9\x1e\x05\x1e\xe8\xf7\xfb\xf2\xfe\xfd{I\x92\xc4\xf6(j\x10\xac_`\x8c\x91\xf5z-\xfb\xfd\xde\xf6(\xf0\xd0d2\x91\xd9l\xc6I\xa8\xbf\x80`\xfd\x874Me\xb5ZY\xbb]0\xda\xa1\xdb\xed\xca|>\xe7\x88\xe2\x7f X?P\xd7\xb5\xacV+9\x1e\x8f\xb6GA\x8b\x8cF#\x99\xcf\xe7\xdc)\xe2\x07\x08\xd6w\x1c\x0e\x07Y\xadV\xad\xbcs\'\xec\x0b\xc3P\xe6\xf39\xa7A|\x07\xd7\x10\xfc\x9f\xd5j%\xbb\xdd\xce\xf6\x18h1c\x8c<<<HQ\x142\x9f\xcfm\x8f\xe3\x14\x82\xf5/c\x8c\xfc\xfd\xf7\xdf\x9c\xa5\x0eg\xecv;)\x8aB>}\xfa\xc4\x0e\xf9\x7f\xf1\x91P\xfe\xb9\xf6o\xb1X\xb0c\x1dN\xeav\xbb\xf2\xf1\xe3G\xaeM\x14\xbe\xaa^\xd24\x95\xfb\xfb{b\x05g\x9d\xcfg\xb9\xbf\xbf\x974Mm\x8fb]\xab\x83\xb5\xdb\xedd\xb1X\xb0s\x1d\xce3\xc6\xc8b\xb1h\xfd\xfe\xd5\xd6\x06k\xb7\xdb\xc9j\xb5\xb2=\x06\xf0[\xda~P\xa8\x95\xc1z>m\x01\xd0h\xb5Z\xc9\xe1p\xb0=\x86\x15\xad\x0bV\x96e\xf2\xf0\xf0`{\x0c\xe0\x8f<<<\xb4\xf2\x9a\xd6V\x05\xab,KY,\x16\xb6\xc7\x00^\xc5b\xb1h\xdd\x1dCZ\x13\xac\xa6id\xb9\\\xb2\x83\x1d\xde0\xc6\xc8r\xb9l\xd5m\xb8[\x13,\xee_\x05\x1f\x95e)\xeb\xf5\xda\xf6\x18W\xd3\x8a`eY\xd6\xea#+\xf0\xdbn\xb7k\xcd\xfe\xacV\x04\x8b#\x82\xf0][\x9e\xe3\xde\x07k\xb7\xdb\xf1Q\x10\xde+\xcb\xb2\x15\x9f"\xbc\x0eV\xd34\xb2\xd9ll\x8f\x01\\\xc5f\xb3\xf1~\x07\xbc\xd7\xc1:\x1e\x8f|\xe32Z\xa3\xaek\xefo8\xe9u\xb0\xda\xf0\x16\x19\xf8\x92\xef\xcfyo\x83U\x96\xa5\x14Ea{\x0c\xe0\xaa\x8a\xa2\xf0z\x9f\xad\xb7\xc1\xe2V\x1ch+\x9f\x9f\xfb\xde\x06\xebt:\xd9\x1e\x01\xb0\xc2\xe7\xe7\xbe\xb7\xc1\xe2\xe3 \xda\xca\xe7\xe7\xbe\x97\xc1\xaa\xeb\xda\xfb\xc3\xbb\xc0\x8f4M\xe3\xed\xd1qo\x83\x05\xb4\x99\xaf\xdb\x80\x97\xc1\x02\xe0\'/\x83\xc5-d\xd0v\xben\x03^\x06\x8b\xfdWh;_\xb7\x01/\x83\x15\x04\x81\xed\x11\x00\xab|\xdd\x06\xbc\x0c\x16\x00?\x11,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x10,\x00j\x04\x9b\xcd\xc6\xbb/03\xc6HY\x96\xb6\xc7\x00\xac\x89\xe3X\xc2\xd0\xbf\xf7#]\xdb\x03\xbc\x850\x0c%I\x12\xdbc\x00xe\xfe%\x18\x80\xb7\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\x08\x16\x005\xfe\x07\xc7\x90\x82NM<\xd04\x00\x00\x00\x00IEND\xaeB`\x82'

# %% ../../nbs/utils/Image.ipynb 12
domo_default_img = Image.from_bytestr(default_img_bytes)


def are_same_image(image1, image2):
    try:
        img_chop = PIL.ImageChops.difference(image1, image2)

        print(np.sum(np.array(img_chop.getdata())))

        if np.sum(np.array(img_chop.getdata())) == 0:
            return True
        return False

    except ValueError as e:
        print(e)
        return False



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/grant.ipynb.

# %% auto 0
__all__ = ["GetGrants_Error", "get_grants"]

# %% ../../nbs/routes/grant.ipynb 3
import httpx

import domolibrary.client.get_data as gd
import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.DomoAuth as dmda
import domolibrary.client.DomoError as de


# %% ../../nbs/routes/grant.ipynb 4
class GetGrants_Error(de.DomoError):
    def __init__(self, status, message, domo_instance, function_name):
        super().__init__(
            status=status,
            message=message,
            domo_instance=domo_instance,
            function_name=function_name,
        )


# %% ../../nbs/routes/grant.ipynb 6
async def get_grants(
    auth: dmda.DomoAuth, debug_api: bool = False, session: httpx.AsyncClient = None
) -> rgd.ResponseGetData:

    url = f"https://{auth.domo_instance}.domo.com/api/authorization/v1/authorities"

    res = await gd.get_data(
        auth=auth, url=url, method="GET", debug_api=debug_api, session=session
    )

    if not res.is_success:
        raise GetGrants_Error(
            status=res.status,
            message=res.message,
            domo_instance=auth.domo_instance,
            function_name="get_grants",
        )

    if len(res.response) == 0:
        raise GetGrants_Error(
            status=res.status,
            message=f"{len(res.response)} grants returned",
            domo_instance=auth.domo_instance,
            function_name="get_grants",
        )

    return res
